//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package store

import (
	//	"encoding/binary"
	"errors"
	"strconv"
	"time"
)

const (
	// well-known bucket names
	infoBn   = "_info"
	groupsBn = "groups"

	publicGroupBn = "_public"

	filesBn     = "files"
	playlistsBn = "playlists"

	// well-known keys
	storeVersionKey = "Version"
)

var (
	groupBuckets = []string{filesBn, playlistsBn} // these buckets will be created inside every group bucket

	ErrNotImplented = errors.New("not implemented")
	ErrNotFound     = errors.New("not found")
	ErrFileInUse    = errors.New("file is in use by at least one playlist")

	InvalidID = uint64(0)
)

// TODO: use conversion to binary because it is faster...
func itob(v uint64) []byte {
	s := strconv.FormatUint(v, 10)
	return []byte(s)
}

// func itob(v uint64) []byte {
// 	b := make([]byte, 8)
// 	binary.BigEndian.PutUint64(b, uint64(v))
// 	return b
// }

func btoi(b []byte) uint64 {
	// ... and parsing of binary can not fail!!
	i, _ := strconv.ParseUint(string(b), 10, 64)
	return i
}

// func btoi(b []byte) uint64 {
// 	return binary.BigEndian.Uint64(b)
// }
// end TODO

type TimeAndUser struct {
	User      string    `json:"user"`
	Timestamp time.Time `josn:"timestamp"`
}

// stored in filesBn
type FileSource struct {
	FileName string `json:"filename"`
	Hash     string `json:"hash"`
}

type FileMetadata struct {
	Artist string `json:"artist"`
	Title  string `json:"title"`
	Album  string `json:"album"`
	// TODO: add more fields
}

type File struct {
	Created     TimeAndUser  `json:"created"`
	LastChanged TimeAndUser  `json:"last-changed"`
	Source      FileSource   `json:"source"`
	Metadata    FileMetadata `json:"metadata"`
	Size        uint64       `json:"size"`
	UsedBy      []uint64     `json:"used-by"`
}

func NewFile(user string) (f *File) {
	f = &File{}
	f.Created.User = user
	f.Created.Timestamp = time.Now()
	return
}

func (f *File) Changed(user string) {
	f.LastChanged.User = user
	f.LastChanged.Timestamp = time.Now()
}

type Files map[uint64]File

// stored in filesBn
type Playlist struct {
	Created     TimeAndUser `json:"created"`
	LastChanged TimeAndUser `json:"last-changed"`
	// TODO: playlist storage layout
}

func NewPlaylist(user string) (p *Playlist) {
	p = &Playlist{}
	p.Created.User = user
	p.Created.Timestamp = time.Now()
	return
}

func (p *Playlist) Changed(user string) {
	p.LastChanged.User = user
	p.LastChanged.Timestamp = time.Now()
}

type Playlists map[uint64]Playlist
